import {
	JsonController,
	Param,
	Body,
	Get,
	Post,
	Put,
	Delete,
	NotFoundError,
	OnUndefined,
	NotAcceptableError,
	Authorized
} from 'routing-controllers';
import { getConnectionManager, Repository } from 'typeorm';
import { EntityFromBody } from 'typeorm-routing-controllers-extensions';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { Runner } from '../models/Runner';
import { IsEmail, IsInt, IsNotEmpty, IsOptional, IsPhoneNumber, IsPositive, IsString } from 'class-validator';
import { RunnerGroupNeededError, RunnerGroupNotFoundError, RunnerIdsNotMatchingError, RunnerNotFoundError, RunnerOnlyOneGroupAllowedError } from '../errors/RunnerErrors';
import { RunnerGroup } from '../models/RunnerGroup';
import { RunnerOrganisation } from '../models/RunnerOrganisation';
import { RunnerTeam } from '../models/RunnerTeam';

class CreateRunner {
	/**
	 * The runners's first name.
	 */
	@IsNotEmpty()
	@IsString()
	firstname: string;

	/**
   * The runners's middle name.
   * Optional
   */
	@IsOptional()
	@IsString()
	middlename?: string;

	/**
   * The runers's last name.
   */
	@IsString()
	@IsNotEmpty()
	lastname: string;

	/**
   * The runner's phone number.
   * Optional
   */
	@IsOptional()
	@IsPhoneNumber('DE')
	phone?: string;

	/**
   * The runner's email address.
   * Optional
   */
	@IsOptional()
	@IsEmail()
	email?: string;

	@IsInt()
	@IsOptional()
	teamId?: number;

	@IsInt()
	@IsOptional()
	orgId?: number;
}

@JsonController('/runners')
//@Authorized('RUNNERS:read')
export class RunnerController {
	private runnerRepository: Repository<Runner>;

	/**
	 * Gets the repository of this controller's model/entity.
	 */
	constructor() {
		this.runnerRepository = getConnectionManager().get().getRepository(Runner);
	}

	@Get()
	@ResponseSchema(Runner, { isArray: true })
	@OpenAPI({ description: 'Lists all runners.' })
	getAll() {
		return this.runnerRepository.find();
	}

	@Get('/:id')
	@ResponseSchema(Runner)
	@ResponseSchema(RunnerNotFoundError, { statusCode: 404 })
	@OnUndefined(RunnerNotFoundError)
	@OpenAPI({ description: 'Returns a runner of a specified id (if it exists)' })
	getOne(@Param('id') id: number) {
		return this.runnerRepository.findOne({ id: id });
	}

	@Post()
	@ResponseSchema(Runner)
	@OpenAPI({ description: 'Create a new runner object (id will be generated automagicly).' })
	async post(
		@Body({ validate: true })
		createRunner: CreateRunner
	) {
		let group: RunnerGroup;

		if(createRunner.teamId && createRunner.orgId){
			throw new RunnerOnlyOneGroupAllowedError();
		}
		if(!createRunner.teamId && !createRunner.orgId){
			throw new RunnerGroupNeededError();
		}

		if(createRunner.teamId){
			group = await getConnectionManager().get().getRepository(RunnerTeam).findOne({ id: createRunner.teamId });
		}
		if(createRunner.orgId){
			group = await getConnectionManager().get().getRepository(RunnerOrganisation).findOne({ id: createRunner.orgId });
		}
		if(!group){
			throw new RunnerGroupNotFoundError();
		}

		delete createRunner.teamId;
		delete createRunner.orgId;
		let runner = <Runner>createRunner;
		runner.group=group;
		console.log(runner)

		return this.runnerRepository.save(runner);
	}

	@Put('/:id')
	@ResponseSchema(Runner)
	@ResponseSchema(RunnerNotFoundError, { statusCode: 404 })
	@ResponseSchema(RunnerIdsNotMatchingError, { statusCode: 406 })
	@OpenAPI({ description: "Update a runner object (id can't be changed)." })
	async put(@Param('id') id: number, @EntityFromBody() runner: Runner) {
		let oldRunner = await this.runnerRepository.findOne({ id: id });

		if (!oldRunner) {
			throw new RunnerNotFoundError();
		}

		if (oldRunner.id != runner.id) {
			throw new RunnerIdsNotMatchingError();
		}

		await this.runnerRepository.update(oldRunner, runner);
		return runner;
	}

	@Delete('/:id')
	@ResponseSchema(Runner)
	@ResponseSchema(RunnerNotFoundError, { statusCode: 404 })
	@OpenAPI({ description: 'Delete a specified runner (if it exists).' })
	async remove(@Param('id') id: number) {
		let runner = await this.runnerRepository.findOne({ id: id });

		if (!runner) {
			throw new RunnerNotFoundError();
		}

		await this.runnerRepository.delete(runner);
		return runner;
	}
}
